#ifdef GL_ES
precision highp float;
#endif

uniform LOWP sampler2D   sampler;
#if defined(ENGINE_OS_ANDROID)
	uniform LOWP sampler2D sampler1;
	uniform int CombinedTexture;
#endif

uniform float u_time;         // u_time умножено в коде на 3.0
uniform LOWP  float u_intensity;

varying vec2        v_texcoord;
varying LOWP vec4   v_color;
varying vec4  screenCoord;

void main()
{
	LOWP vec4 tex = texture2D(sampler, v_texcoord);
#if defined(ENGINE_OS_ANDROID)
	if (CombinedTexture == 1){
		tex.a = texture2D(sampler1, v_texcoord).r;
	}
#endif
	MEDIUMP float causticSim = sin(tex.g * screenCoord.x * screenCoord.y + u_time) * u_intensity + u_intensity;

	// освещенность
	MEDIUMP float lightness = min(1.0, (tex.r + tex.g + tex.b) * 0.5); // 0.333 * 1.5

	// выделяем некий средний "слой"
	// в этот слой не попадают темные и самые яркие фрагменты
	lightness = lightness * lightness * (1.0 - lightness);
	
	// что эквивалентно
	// lightness = pow(lightness, 2.0) - pow(lightness, 3.0);
	
	//lightness = clamp(lightness, 0.0, 1.0);
	
	// fade-in каустики от верха экрана до screenCoord.w
	MEDIUMP float vertSkip = 1.0 - clamp((screenCoord.z - screenCoord.w) / (1.0 - screenCoord.w), 0.0, 1.0);
	
	LOWP vec4 color = v_color * tex + causticSim * tex.a * lightness * vertSkip;
	gl_FragColor = vec4(color.rgb, tex.a);
	
	// просмотр маски lightness
	// gl_FragColor.rgb = vec3(lightness, lightness, lightness);
}
