#ifdef GL_ES
precision highp float;
#endif

uniform MEDIUMP sampler2D sampler;
uniform MEDIUMP sampler2D sampler1;
uniform MEDIUMP sampler2D sampler2;
varying MEDIUMP vec2 v_texcoord;
varying MEDIUMP vec2 v_texcoord1;
varying MEDIUMP vec2 v_texcoordLightmap;

uniform LOWP vec3 u_fogColor;
varying float v_density;

uniform LOWP vec3 v_portholeColor;
varying LOWP vec3 v_porthole;

void main()
{
	LOWP vec4 diffuse_color = texture2D(sampler, v_texcoord);
	LOWP vec4 mask = texture2D(sampler1, vec2(0.0, 0.5) + v_texcoord * 0.5);
	LOWP vec4 lightmap_color = texture2D(sampler2, v_texcoordLightmap);

	LOWP float mask_hull    = mask.r;
	LOWP float mask_glass   = mask.g;
	LOWP float mask_orange  = mask.b;
	LOWP float mask_screw   = step(0.5, mask.a);
	LOWP float mask_lighter = step(0.2, (1.0 - mask.a) * mask.a);
	LOWP float mask_frame   = 1.0 - step(0.03, mask.r + mask.g + mask.b + mask_screw + mask_lighter);

	LOWP vec4 matcap_color_glass  = texture2D(sampler1, vec2(0.0, 0.75) + v_texcoord1 * vec2(0.5, 0.25));
	LOWP vec4 matcap_color_hull   = texture2D(sampler1, vec2(0.0, 0.5) + v_texcoord1 * vec2(0.5, 0.25));
	LOWP vec4 matcap_color_frame  = texture2D(sampler1, vec2(0.5, 0.0) + v_texcoord1 * 0.5);
	LOWP vec4 matcap_color_metall = texture2D(sampler1, vec2(0.5, 0.5) + v_texcoord1 * 0.5);
	LOWP vec4 matcap_color_orange = vec4(matcap_color_glass.a, matcap_color_hull.a, 0.0, 0.0);

	MEDIUMP vec3 color_hull = mix(diffuse_color.rgb, vec3(0.7843, 0.3921, 0.0), matcap_color_hull.r) + vec3(1.0, 0.9450, 0.6980)*(matcap_color_hull.g+matcap_color_hull.a);
	
	MEDIUMP vec3 color_glass = mix(diffuse_color.rgb,
								  (diffuse_color.rgb * mix(0.6, 1.4, matcap_color_glass.r) + matcap_color_glass.ggg),
								  1.0 - matcap_color_glass.b);
	// color_glass = mix(color_glass, v_portholeColor + matcap_color_glass.ggg, v_porthole.x);
	// color_glass = mix(color_glass, v_portholeColor + matcap_color_glass.ggg, v_porthole.y);
	// color_glass = mix(color_glass, v_portholeColor + matcap_color_glass.ggg, v_porthole.z);
	color_glass = mix(color_glass, v_portholeColor + matcap_color_glass.ggg, v_porthole.x + v_porthole.y + v_porthole.z);

	MEDIUMP vec3 color_orange = mix(
		diffuse_color.rgb + vec3(1.0, 0.65, 0.35) * matcap_color_orange.r,
		vec3(0.2470, 0.1647, 0.5058),
		matcap_color_hull.r*0.5);
	MEDIUMP vec3 color_screw = mix(diffuse_color.rgb, vec3(1.0, 0.5, 0.0), matcap_color_glass.g)+matcap_color_hull.a;
	MEDIUMP vec3 color_lighter = diffuse_color.rgb;
	MEDIUMP vec3 color_frame = mix(diffuse_color.rgb, matcap_color_frame.rgb, matcap_color_frame.a) + matcap_color_orange.g;

	// маска для смешивания карты освещенности
	float lightmap_mix = length((v_texcoord1 - vec2(0.5, 0.5)) * 2.0);

	
	MEDIUMP vec4 color = vec4(diffuse_color.rgb *
		clamp(1.0 - (mask_hull + mask_glass + mask_orange + mask_screw + mask_lighter + mask_frame), 0.0, 1.0) +
		color_hull * mask_hull +
		color_glass * mask_glass +
		color_orange * mask_orange +
		color_screw * mask_screw +
		color_lighter * mask_lighter +
		color_frame * mask_frame,
		1.0);


	color = mix(
		color,
		mix(color,
			max(color, lightmap_color) * vec4(0.3, 0.3, 0.3, 0.3) + 0.7 * mix(color, lightmap_color * 1.8, lightmap_mix),
			0.5),
		lightmap_mix);
		

	color.rgb -= (color.rgb - u_fogColor) * v_density;

	
	gl_FragColor = color;
}
