const int MAX_BONES = 80;

attribute vec4 a_vertex;
attribute vec3 a_normal;
attribute MEDIUMP vec2 a_texcoord;
attribute vec4 a_weights;
attribute vec4 a_indices;

uniform mat4 mViewProj;
uniform vec4 mBonesArray[MAX_BONES * 3];

uniform vec4 u_lightPos;
uniform MEDIUMP vec2 u_fogBorder;
uniform vec3 u_cameraUp;
uniform vec2 u_projLightmapCenter;  // центр проекции батискафа на карте освещенности
uniform vec2 u_lightmapDeltaUV;  // половинная ширина, высота прямоугольника карты освещенности, попадающего на батискаф

varying float v_density;

uniform mat4 u_view;

varying MEDIUMP vec2 v_texcoord;
varying MEDIUMP vec2 v_texcoord1;
varying MEDIUMP vec2 v_texcoordLightmap;

uniform LOWP vec3 v_portholeBrightness;
varying LOWP vec3 v_porthole;

void main()
{
	int i = int(a_indices.x) * 3;
	HIGHP vec3 p = vec3(dot(mBonesArray[i], a_vertex), dot(mBonesArray[i+1], a_vertex), dot(mBonesArray[i+2], a_vertex)) * a_weights.x;
	
	HIGHP vec4 a_normal_fixed = vec4(a_normal, 0);
	HIGHP vec3 pn = vec3(dot(mBonesArray[i], a_normal_fixed), dot(mBonesArray[i+1], a_normal_fixed), dot(mBonesArray[i+2], a_normal_fixed)) * a_weights.x;
	
	i = int(a_indices.y) * 3;
	p += vec3(dot(mBonesArray[i], a_vertex), dot(mBonesArray[i+1], a_vertex), dot(mBonesArray[i+2], a_vertex)) * a_weights.y;
	pn += vec3(dot(mBonesArray[i], a_normal_fixed), dot(mBonesArray[i+1], a_normal_fixed), dot(mBonesArray[i+2], a_normal_fixed)) * a_weights.y;

	i = int(a_indices.z) * 3;
	p += vec3(dot(mBonesArray[i], a_vertex), dot(mBonesArray[i+1], a_vertex), dot(mBonesArray[i+2], a_vertex)) * a_weights.z;
	pn += vec3(dot(mBonesArray[i], a_normal_fixed), dot(mBonesArray[i+1], a_normal_fixed), dot(mBonesArray[i+2], a_normal_fixed)) * a_weights.z;

	i = int(a_indices.w) * 3;
	p += vec3(dot(mBonesArray[i], a_vertex), dot(mBonesArray[i+1], a_vertex), dot(mBonesArray[i+2], a_vertex)) * a_weights.w;
	pn += vec3(dot(mBonesArray[i], a_normal_fixed), dot(mBonesArray[i+1], a_normal_fixed), dot(mBonesArray[i+2], a_normal_fixed)) * a_weights.w;

	gl_Position = mViewProj * vec4(p, 1.0);

	LOWP vec3 normal = normalize(pn);

	LOWP float u = dot(vec3(1.0, 0.0, 0.0), normal);
	LOWP float v = dot(u_cameraUp, normal);
	
	v_texcoord = a_texcoord;
	v_texcoord1 = vec2((u + 1.0), (v + 1.0)) * 0.5;
	v_texcoordLightmap = u_projLightmapCenter + vec2(u_lightmapDeltaUV.x * u, u_lightmapDeltaUV.y * v);
	
	// туман
	float d = length(vec4(p, 1.0) - u_lightPos);
	float f = (u_fogBorder.y - d) / (u_fogBorder.y - u_fogBorder.x);
	v_density = (1.0 - clamp(f, 0.0, 1.0)) / 1.3;
	
	// находим окна по геометрическим признакам
	v_porthole.y = min(1.0, max(0.0, a_vertex.x - 6.0));
	v_porthole.x = (1.0 - v_porthole.y) * clamp(-3.0 - a_vertex.z, 0.0, 1.0);
	v_porthole.z = (1.0 - v_porthole.y) * clamp(-3.0 + a_vertex.z, 0.0, 1.0);
	v_porthole *= v_portholeBrightness;
}
