const int MAX_BONES = 80;

attribute vec4 a_vertex;
attribute MEDIUMP vec2 a_texcoord;
attribute vec4 a_weights;
attribute vec4 a_indices;
attribute vec3 a_normal;

uniform mat4 mViewProj;
uniform vec4 mBonesArray[MAX_BONES*3];
uniform vec4 u_lightPos;
uniform LOWP vec3 u_fogColor;
uniform vec2 u_fogBorder;
uniform float u_wavePos;

varying MEDIUMP vec2 v_texcoord; // размер текстуры max 256х256
varying LOWP vec4 v_color;

varying float v_density;

vec2 PlanarUV(vec4 vertex)
{
	return vec2(vertex.x * 2.0 + 0.5, 0.1 + (vertex.y * 1.25 + 0.25));
}

float PushMask(vec4 v)
{
	return step(v.z, 0.05);
}

float gradient_twoside(float value, float gradient_start, float gradient_range)
{
	float dist_rel = 1.0 - min(distance(value, gradient_start) / gradient_range, 1.0);
	return dist_rel;
}

vec4 push(vec3 v, vec3 n, vec2 planar_uv, float wave_pos, float wave_width, float amount, float amount_fixed, float use_gradient)
{
	vec4 res = vec4(v.x, v.y, v.z, 0.0);

	float wave = gradient_twoside(fract(planar_uv.y), wave_pos + v.z * 5.5, wave_width);
	wave = mix(wave, wave * pow(planar_uv.y, 1.5), use_gradient);

	res.xyz += n.xyz * wave * 0.1 * amount + n.xyz * 0.01 * amount_fixed;
	res.w = wave;

	return res;
}


void main()
{
	float _WaveWidth = 1.0;
	float _Push = 1.0;
	
	
	v_texcoord = a_texcoord;
	
	vec2 uv_planar = PlanarUV(a_vertex);
	float push_mask = PushMask(a_vertex);

	// выдавливаем ската
	// vec4 v_ = vec3 pos = push(a_vertex.xyz, a_normal.xyz, uv_planar, u_wavePos, _WaveWidth, push_mask * _Push, 0, 0.5); 
	//v.vertex.xyz = v_.xyz;
	vec4 pos;
	//pos = push(a_vertex.xyz, a_normal.xyz, uv_planar, u_wavePos, _WaveWidth, push_mask * _Push, 0.0, 0.5);
	pos.xyz = a_vertex.xyz;
	pos.w = a_vertex.w;

	int i = int(a_indices.x) * 3;
	vec3 p = vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.x;
	if (a_weights.y > 0.0) {
		i = int(a_indices.y) * 3;
		p += vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.y;
	}
	if (a_weights.z > 0.0) {
		i = int(a_indices.z) * 3;
		p += vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.z;
	}
	if (a_weights.w > 0.0) {
		i = int(a_indices.w) * 3;
		p += vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.w;
	}
	gl_Position = mViewProj * vec4(p, 1.0);
	
	float d = length(vec4(p, 1.0) - u_lightPos);
	float f = (u_fogBorder.y - d) / (u_fogBorder.y - u_fogBorder.x);
	v_density = (1.0 - clamp(f, 0.0, 1.0)) / 1.3;

	v_color = vec4(1.0);
}
