const int MAX_BONES = 80;

attribute vec4 a_vertex;
attribute MEDIUMP vec2 a_texcoord;
attribute vec4 a_weights;
attribute vec4 a_indices;
attribute vec3 a_normal;

uniform mat4 mViewProj;
uniform vec4 mBonesArray[MAX_BONES*3];
uniform vec4 u_lightPos;
uniform vec2 u_fogBorder;
uniform float u_wavePos;

varying MEDIUMP vec2 v_texcoord;
varying vec2 v_uv_noise;
varying LOWP vec4 v_color;
varying float v_density;
//varying float v_tail;

vec2 PlanarUV(vec4 vertex)
{
    const float scale = 1.0;

#ifdef UV_DEBUG
    return vec2(
		vertex.x * scale,
        (vertex.z * scale) + 20.0
        );
#endif

    return vec2(
        (vertex.x * scale) / 42.98 + 0.5,
        (((vertex.z * scale) + 20.0) / 60.0)
        );
}

float PushMask(vec4 v)
{
	return step(v.y, 8.0);
}

float gradient_twoside(float value, float gradient_start, float gradient_range)
{
	float dist_rel = 1.0 - min(distance(value, gradient_start) / gradient_range, 1.0);
	return dist_rel;
}

vec4 push(vec3 v, vec3 n, vec2 planar_uv, float wave_pos, float wave_width, float amount, float amount_fixed, float use_gradient)
{
	vec4 res = vec4(v.x, v.y, v.z, 0.0);

	float wave = gradient_twoside(fract(planar_uv.y), wave_pos, wave_width);
	wave = mix(wave, wave * pow(planar_uv.y, 1.5), use_gradient);

	res.xyz += n.xyz * wave * 0.1 * amount * 80.0 + n.xyz * 0.01 * amount_fixed * 80.0;
	res.w = wave;

	return res;
}

void main()
{
	float _WaveWidth = 0.407;
	float _Push = 0.15;
	float _NoiseScale = 0.1;
	
	vec2 uv_planar = PlanarUV(a_vertex);
	float push_mask = PushMask(a_vertex);

	// выдавливаем ската
	vec4 pushedPos = push(a_vertex.xyz, a_normal.xyz, uv_planar, u_wavePos, _WaveWidth, push_mask * _Push, 0.5, 0.0);
	vec4 pos = pushedPos;
	pos.w = a_vertex.w;
	
	// наличие волны
	v_color.r = clamp(pushedPos.w * push_mask * 2.0, 0.0, 1.0);
	
	// маркер хвоста
	float tail = clamp((uv_planar.y - 0.5) * 2.0, 0.0, 1.0);
	v_color.g = tail;

	// прозрачность боковых частей
	//v_color.a = abs(dot(a_normal.xyz, vec3(0.0, 1.0, 0.0)));

	int i = int(a_indices.x) * 3;
	vec3 p = vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.x;
	if (a_weights.y > 0.0) {
		i = int(a_indices.y) * 3;
		p += vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.y;
	}
	if (a_weights.z > 0.0) {
		i = int(a_indices.z) * 3;
		p += vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.z;
	}
	if (a_weights.w > 0.0) {
		i = int(a_indices.w) * 3;
		p += vec3(dot(mBonesArray[i], pos), dot(mBonesArray[i+1], pos), dot(mBonesArray[i+2], pos)) * a_weights.w;
	}
	gl_Position = mViewProj * vec4(p, 1.0);
	
	v_texcoord = uv_planar;
	v_uv_noise = uv_planar * 2.0 * _NoiseScale;

	float d = length(vec4(p, 1.0) - u_lightPos);
	float f = (u_fogBorder.y - d) / (u_fogBorder.y - u_fogBorder.x);
	v_density = (1.0 - clamp(f, 0.0, 1.0)) / 1.3;
}