#ifdef GL_ES
precision highp float;
#endif

uniform LOWP vec3 u_fogColor;
uniform LOWP vec3 u_toneColor;
uniform MEDIUMP float u_density;
uniform MEDIUMP float u_causticFogScale;
uniform MEDIUMP float u_causticRange;
uniform MEDIUMP float u_causticPower;

uniform LOWP sampler2D sampler;
uniform LOWP sampler2D sampler2;

#if defined(ENGINE_OS_ANDROID)
	uniform LOWP sampler2D sampler1;
	uniform int CombinedTexture;
#endif

varying LOWP vec4 v_color;
varying vec2 v_texcoord;
varying MEDIUMP vec2 v_causticTexCoord;

MEDIUMP float Brightness(vec3 color)
{
	return 0.333 * (color.r + color.g + color.b);
}

void main()
{
	LOWP vec4 texColor = v_color * texture2D(sampler, v_texcoord);
	LOWP vec3 color = texColor.rgb;
	
	LOWP vec3 toneColor = u_toneColor - (u_toneColor - u_fogColor) * u_density;

	//Окрашиваем тени в цвет тумана с сохранением прежних значений яркости пикселей
	//Степень окрашивания зависит от яркости пикселя (чем темнее, тем сильнее окрашиваем)
	float brightness = Brightness(color);
	color -= (color - color * toneColor) * (1.0 - brightness);
	//Определяем отклонение новой яркости от прежней яркости и устраняем его
	color *= brightness / max(0.01, Brightness(color));
	brightness = Brightness(color);

	// До u_causticRange яркость каустики нулевая, далее линейно увеличивается до центра между u_causticRange и 1
	// после этой точки ограничена 1
	MEDIUMP float rangeScale = clamp(2.0 * (brightness - u_causticRange) / (1.0 - u_causticRange), 0.0, 1.0);

	// Не накладываем каустику на яркие пискели чтобы избежать пересвеченных областей
	// Линейно падает от 1 при яркости 0.75 до 0 при яркости 1
	MEDIUMP float overbrightScale = clamp(4.0 - 4.0 * brightness, 0.0, 1.0);

	LOWP vec3 caustic = texture2D(sampler2, v_causticTexCoord).rgb * u_causticPower * rangeScale * overbrightScale * u_causticFogScale;
	color += caustic;

	#if defined(ENGINE_OS_ANDROID)
		if (CombinedTexture == 1) {
			LOWP vec4 alpha = texture2D(sampler1, v_texcoord);
			texColor.a = v_color.a * alpha.r;
		}
	#endif

	color -= (color - u_fogColor) * u_density;
	gl_FragColor = vec4(color, texColor.a);
}