#ifdef GL_ES
precision highp float;
#endif

uniform LOWP sampler2D sampler;
uniform LOWP sampler2D sampler1;

varying MEDIUMP vec2 v_texcoord; 
varying MEDIUMP vec2 v_capcoord;

void main()
{
	vec4 diffuse_color = texture2D(sampler, v_texcoord);
	
	//6 вспомогательных текстур упакованы в одно RGBA изображение. 
	//Изображение 256х256px разбито на квадраты делением по полам, 3 текстуры занимают 3/4 площади (квадраты по 128х128px), и еще 3 текстуры упакованы в 1/4 и занимают в итоге по 1/8 площади (квадраты 64x64px).
	
	//Читаем цвет маски дерево/металл из вспомогательной текстуры (левый нижний малый квадрат)
	float materials_mix = texture2D(sampler1, v_texcoord* 0.25 - vec2(0.00,0.25)).r;

	//Вычислим заранее текстурные координаты для всех вспомогательных текстур
	vec2 chan1_uv = vec2(0.00, 0.50) + v_capcoord * 0.50;  //левый верхний большой квадрат (matcap текстура отражения для металлической части молотка)
	vec2 chan2_uv = vec2(0.50, 0.50) + v_capcoord * 0.50; //правый верхний большой квадрат (matcap текстура для материала дерева)
	vec2 chan3_uv = vec2(0.00, 0.25) + v_capcoord * 0.25; //левый верхний малый квадрат (matcap текстура для эффекта отражения по Френелю)
	vec2 chan4_uv = vec2(0.50, 0.00) + v_texcoord * 0.50 - vec2(0.00, 0.50); //правый нижний большой квадрат (базовая текстура для gloss эффекта на ребрах металлической части)

	//Читаем цвет из карты отражения по Френелю, будем использовать как фактор смешивания базовой текстуры молотка и matcap текстуры отражения для металлической части)
	float fresnel = texture2D(sampler1, chan3_uv).r;
	//Читаем цвет из правого нижнего малого квадрата - это вспомогательная текстура для gloss эффекта на ребрах металлической части, используем для определения силы gloss эффекта в зависимости от ориентации молотка к наблюдателю
	float gloss = texture2D(sampler1, chan3_uv + vec2(0.25, -0.25)).r;

	//Читаем текстуру отражения металлической части, либо текстуру материала дерева (в зависимости от маски дерево/металл)
	vec4 multi_matcap_color = mix(texture2D(sampler1, chan1_uv), 
									texture2D(sampler1, chan2_uv),
									 materials_mix);

	//Вычисляем цвет металлической части (используем карту отражения по Френелю + корректировки для усиления яркости)								 
	vec3 metal_material = mix(diffuse_color.rgb, multi_matcap_color.rgb * 1.2, fresnel);
	//Вычисляем цвет деревянной части (используем наложение по альфе + корректировки для усиления яркости)
	vec3 wood_material  = diffuse_color.rgb; //mix(diffuse_color.rgb * 1.35, multi_matcap_color.rgb * 1.1, multi_matcap_color.a);

	//Вычисляем финальный цвет (с помощью маски дерево/металл выбираем нужный материал и складываем с текстурой из правого нижнего квадрата с домножением на силу gloss эффекта)
	gl_FragColor.rgb = mix(metal_material, wood_material, materials_mix) + texture2D(sampler1, chan4_uv).rgb * gloss;
	gl_FragColor.a = 1.0;
}
