#ifdef GL_ES
precision highp float;
#endif

const int blurQuality = 3;

uniform MEDIUMP float u_blurSize;
uniform LOWP vec2 u_direction;
uniform HIGHP vec2 u_pixelSize;
uniform LOWP sampler2D sampler;

#if defined(ENGINE_OS_ANDROID)
	uniform LOWP sampler2D sampler1;
	uniform int CombinedTexture;
#endif

varying LOWP vec4 v_color;
varying HIGHP vec2 v_texcoord;

float clampOutTex(vec2 texcoord)
{
	return step(0.0, texcoord.x) * step(texcoord.x, 1.0) * step(0.0, texcoord.y) * step(texcoord.y, 1.0);
}

float clampAlphaBorder(vec2 texcoord, vec2 border)
{
	vec2 center = vec2(0.5, 0.5);
	//vec2 border = 10.0*u_blurSize * u_pixelSize;
	vec2 alpha = center - vec2(abs(center.x - texcoord.x), abs(center.y - texcoord.y));
	alpha *= center / border;
	return clamp(alpha.x, 0.0, 1.0) * clamp(alpha.y, 0.0, 1.0);
}

void main()
{
	MEDIUMP float alphaSum = texture2D(sampler, v_texcoord).a;
	HIGHP vec2 blurStep = u_direction * u_pixelSize;
	blurStep *= u_blurSize / float(blurQuality);
	MEDIUMP vec2 texcoord1 = v_texcoord;
	MEDIUMP vec2 texcoord2 = v_texcoord;
	for (int i = 0; i < blurQuality; ++i) {
		texcoord1 += blurStep;
		texcoord2 -= blurStep;
		alphaSum += clampOutTex(texcoord1) * texture2D(sampler, texcoord1).a;
		alphaSum += clampOutTex(texcoord2) * texture2D(sampler, texcoord2).a;
	}
	
	alphaSum *= clampAlphaBorder(v_texcoord, 2.0*u_blurSize * u_pixelSize) / float(2 * blurQuality + 1);
	gl_FragColor = vec4(1.0, 1.0, 1.0, alphaSum);
}