#ifdef GL_ES
precision highp float;
#endif

#define PI 3.14159265359

varying vec2 v_texcoord;

/*
Диагональная подсветка на счетчике молний.
p - период подсветки [0..1].
Блик выглядит как диагональная полоса по текстуре.
Ширина линии задается в долях высоты текстуры: 0.3 - блик шириной в треть высоты, 0.1 - в 1/10 высоты и т.д.
h1 - ширина блика сплошного цвета
h2 - ширина блика градиентной части.
h1 < h2
Средняя линия блика: y = Kx + B;
K - тангенс угла наклона блика к Ох.
B - смещение линии. Т.к. блик должен пробежать от левого нижнего до правого верхнего, то B должен измениться в диапазоне [bs..be].
*/

uniform float p;		
uniform float bs;
uniform float be;
uniform float h1;
uniform float h2;

uniform LOWP sampler2D sampler;

#if defined(ENGINE_OS_ANDROID)
	uniform LOWP sampler2D sampler1;
	uniform int CombinedTexture;
#endif

const float K = -2.31;

//Calculate Intensity
float ci(float y, float h)
{
	float i = clamp((v_texcoord.y - y + h)/(h*2.0),0.0,1.0);
	return sin(PI * i);
}

void main(void)
{
	float y = K * v_texcoord.x + mix(bs, be, p) + mix(-h2, h2, p);
	gl_FragColor = texture2D(sampler, v_texcoord);
	gl_FragColor.rgb *= 1.0 + ci(y, h2);
	gl_FragColor.rgb += (1.0 - gl_FragColor.rgb) * ci(y, h1);
#if defined(ENGINE_OS_ANDROID)
	if (CombinedTexture == 1){
		gl_FragColor.a = texture2D(sampler1, v_texcoord).r;
	}
#endif
}