#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D   sampler;
#if defined(ENGINE_OS_ANDROID)
	uniform LOWP sampler2D sampler1;
	uniform int CombinedTexture;
#endif

uniform sampler2D sampler2;
#if defined(ENGINE_OS_ANDROID)
	uniform LOWP sampler2D sampler3;
#endif

uniform float u_time;         // u_time умножено в коде на 3.0
uniform LOWP  float u_intensity;

varying vec2        v_texcoord;
varying LOWP vec4   v_color;
varying vec4  screenCoord;
varying vec4 v_causticsCoord;

void main()
{
	LOWP vec4 tex = texture2D(sampler, v_texcoord);
#if defined(ENGINE_OS_ANDROID)
	if (CombinedTexture == 1){
		tex.a = texture2D(sampler1, v_texcoord).r;
	}
#endif

	vec2 causticsUV = vec2(0.5 + v_causticsCoord.x * v_causticsCoord.y, v_causticsCoord.z * 3.0);
	vec4 caustics = texture2D(sampler2, causticsUV);
#if defined(ENGINE_OS_ANDROID)
	if (CombinedTexture == 1){
		caustics.a = texture2D(sampler3, causticsUV).r;
	}
#endif
	float texCaustVertSkip = clamp((0.8 - 2.0 * screenCoord.z), 0.0, 1.0);

	float causticSim = sin(tex.g * screenCoord.x * screenCoord.y + u_time) * u_intensity;

	// освещенность
	float lightness = min(1.0, (tex.r + tex.g + tex.b) * 0.5);

	// выделяем некий средний "слой"
	// в этот слой не попадают темные и самые яркие фрагменты
	lightness = lightness * lightness * (1.0 - lightness);
	
	float vertSkip = 1.0 - clamp((screenCoord.z - screenCoord.w) / (1.0 - screenCoord.w), 0.0, 1.0);

	gl_FragColor = v_color * tex + causticSim * tex.a * lightness * vertSkip + caustics * texCaustVertSkip;
	
	// просмотр маски lightness
	// gl_FragColor.rgb = vec3(lightness, lightness, lightness);
}
