#if defined(GL_ES) && !defined(LOWP)
#define LOWP lowp
#define MEDIUMP mediump
#define HIGHP highp
#endif

#ifdef GL_ES
precision highp float;
#endif

uniform sampler2D sampler;
uniform sampler2D sampler2;

uniform float u_time;         // u_time is multiplied by _Speed in code
uniform LOWP  float u_intensity;
uniform MEDIUMP float u_maskBrightness;

varying vec2 v_texcoord;
varying vec2 v_causticsCoord;

void main()
{
	MEDIUMP vec4 col;
	
	// NOTE: Both fetches are from Render Targets! No need for separate Texture for Alpha on Android!
	LOWP vec4 tex = texture2D(sampler, v_texcoord);
	LOWP vec4 tex_low = texture2D(sampler2, v_texcoord);

	// caustics
	float coordScale = v_causticsCoord.x * v_causticsCoord.y;
	
	float causticSim = sin(tex_low.b * coordScale + u_time) * u_intensity + u_intensity;

	// lightness of the current region
	float lightness = min(1.0, (tex.r + tex.g + tex.b) * u_maskBrightness);
	// designer's decision
	lightness = lightness * lightness * (1.0 - lightness);

	col.rgb = tex.rgb;
	col.rgb += tex.rgb * causticSim * lightness;

	gl_FragColor = col;
}	

