#ifdef GL_ES
precision highp float;
#endif

//uniform LOWP sampler2D sampler;
//uniform LOWP sampler2D sampler1;
//uniform LOWP sampler2D sampler2;
uniform LOWP sampler2D sampler3; // _NoiseTex
uniform LOWP vec3 u_fogColor;
uniform LOWP float u_blend1;
uniform LOWP float u_blend2;
uniform float u_time;
uniform vec4 u_randomVector;
uniform float u_timeDirection;

varying vec2 v_texcoord;
varying vec2 v_uv_noise;
varying LOWP vec4 v_color;
varying float v_density;
//varying float v_tail;

#ifdef HSB

	vec3 rgb2hsv(vec3 c)
	{
		vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
		vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
		vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

		float d = q.x - min(q.w, q.y);
		float e = 1.0e-10;
		return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
	}

	vec3 hsv2rgb(vec3 c)
	{
		vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
		vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
		return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
	}

#endif

float level(float value, float gradient_range)
{
	float dist = abs(value - 1.0);
	float dist_rel = 1.0 - min(dist / gradient_range, 1.0);
	return dist_rel;
}

#ifdef UV_DEBUG
	
	float gradient_twoside(float value, float gradient_start, float gradient_range)
	{
		float dist_rel = 1.0 - min(distance(value, gradient_start) / gradient_range, 1.0);
		return dist_rel;
	}
	
#endif

void main()
{
	float u_Emission = 0.1;
	float noiseAnimSpeed = 0.4;
	float noiseMoveSpeed = 0.15;
	
	float level0 = 0.025;
	float level1 = 0.08;
	float level2 = 0.25;
	
	const LOWP vec3 color_AAA = vec3(0.8, 0.2, 0.2); // alpha for each layer
	
#ifdef HSB

	//const LOWP vec3 color_HSB = vec3(0.5, 1.0, 1.0); // blue
	const LOWP vec3 color_HSB = vec3(0.1, 0.99, 0.995); // yellow
	
	vec3 color_RGB = hsv2rgb(color_HSB);
	vec4 level0Color;
	level0Color.rgb = color_RGB;
	level0Color.a = color_AAA.r;
	
	vec4 level1Color;
	level1Color.rgb = color_RGB;
	level1Color.a = color_AAA.g;
	
	vec4 level2Color;
	level2Color.rgb = color_RGB;
	level2Color.a = color_AAA.b;
	
#else

	//const LOWP vec3 color_RGB = vec3(0.0, 255.0 / 255.0, 255.0 / 255.0); // blue
	//const LOWP vec3 color_RGB = vec3(255.0 / 255.0, 153.0 / 255.0, 0.0 / 255.0); // yellow
	const LOWP vec3 color_RGB = vec3(1.0, 0.6, 0.0); // yellow
	
	vec4 level0Color;
	level0Color.rgb = color_RGB;
	level0Color.a = color_AAA.r;
	
	vec4 level1Color;
	level1Color.rgb = color_RGB;
	level1Color.a = color_AAA.g;
	
	vec4 level2Color;
	level2Color.rgb = color_RGB;
	level2Color.a = color_AAA.b;
	
#endif

	float scale = 1.6;
	float mask  = v_color.r;
	float tail  = v_color.g * 0.5;
	
	//LOWP vec4 globalColor = vec4(255.0 / 255.0, 255.0 / 255.0, 255.0 / 255.0, 255.0 / 255.0);
	LOWP vec4 globalColor = vec4(1.0, 1.0, 1.0, 1.0);

	////////////////////
	
	vec2 uv0 = v_uv_noise * scale;
	vec2 uv1 = v_uv_noise * 2.5 * scale;
	
	uv0.x += u_time * noiseAnimSpeed;
	uv1.x -= u_time * noiseAnimSpeed;
	
	vec4 n0 = texture2D(sampler3, uv0 - vec2(0.0, u_time * noiseMoveSpeed * scale) * u_timeDirection);
	vec4 n1 = texture2D(sampler3, uv1 - vec2(0.0, u_time * noiseMoveSpeed * scale) * u_timeDirection);
	float noise = (n0 + n1).r;
	
	////////////////////
	
	MEDIUMP vec3 eff = vec3(0.0, 0.0, 0.0);
	
	eff += (level(noise, level0) * level0Color.a * 2.0) * level0Color.rgb * mask;
	eff += (level(noise, level1) * level1Color.a * 2.0) * level0Color.rgb * mask;
	eff += (level(noise, level2) * level2Color.a * 2.0) * level0Color.rgb * mask;
	
	// light on the wave
	//eff += level0Color.rgb * mask * 0.25;
	
	// light on the tail
	eff += level0Color.rgb * tail * mask;
	
	MEDIUMP vec4 col = globalColor;
	col.rgb *= eff;
	
#ifdef UV_DEBUG

	//float q = gradient_twoside(v_texcoord.y, 20.0, 1.0) * 10.0;
	float q = v_texcoord.y;
	col.rgb = vec3(q, q, q);
	col.a = 1.0;
	
#endif
	
	// required for correct work of blending mode BlendMode::Add
	col *= col.a;
	col *= clamp(v_texcoord.y * 2.5, 0.5, 1.0);

	gl_FragColor =  col;
}